c
c
      program Pendulum
c
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
	character*60 fname
	character*7 flag
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
c
c
c     xl       : length of string
c     xmass    : mass of obejectswinging
c     viscc    : coefficient of viscocity
c     theata0  : initial angle displacement
c     tinc     : time increment for calculation
c     nstep    : number of calculation steps
c     iprint   : skip number of output
c                results will be output at step = (j-1)*iprint   j=1, nstep
c     deltan   : delta in Newmark's beta method     
c     betan    : beta in Newmark's beta method 
c     aext     : amplitude of supporting point
c     roeo0    : ratio omegae / omega0
c
c     nfidt    : file number of input data
c     nfthd    : file number of time history      
c
c
      pi = 4.d0 * datan(1.d0)
c
c 
c ---------------------------
c
c
c     / FILE NUMBER /
c
      nfidt=10      ! (nfidt, file='WavePropagation1D.idt')
	nfthd=22      ! (nfidt, file='WavePropagation1D.thd')
c
c     / OPEN FILES /
c
      call inpdat
c
c     / SOLVE /
c
      call solve_main
c
c
      stop 777
c
c
	end

c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   I N P D A T             ********
c*                                                                   *
c*********************************************************************
c
c
	subroutine inpdat
c
c     input parameters and initial conditions from (nfidt, file='Pendulum.idt')
c
      implicit double precision (a-h, o-z)
      implicit integer (i-n)
	character*5 flag
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
c
c
c     
c----------------------------------------------------------------------
c
c
      open (nfidt, file='Pendulum.idt')      !input data   ##
c
c
c
c     / PARAMETRS /
c
c
c     xl       : length of string
c     xmass    : mass of obejectswinging
c     viscc    : coefficient of viscocity
c     theata0  : initial angle displacement
c     tinc     : time increment for calculation
c     nstep    : number of calculation steps
c     iprint   : skip number of output
c                results will be output at step = (j-1)*iprint   j=1, nstep
c     deltan   : delta in Newmark's beta method     
c     betan    : beta in Newmark's beta method 
c     aext     : amplitude of supporting point
c     roeo0    : ratio omegae / omega0
c
c
      grav=9.8d0
c
c
      rewind (nfidt)
  100 read (nfidt, 1000, end=555, err=999) flag
 1000 format(a5)
      if (flag .ne. '/para') goto 100
      read (nfidt, *) xl
      read (nfidt,*) xmass
      read (nfidt,*) viscc
      read (nfidt,*) theata0
      read (nfidt,*) tinc
      read (nfidt,*) nstep
      read (nfidt,*) iprint
      read (nfidt,*) deltan
      read (nfidt,*) betan
      read (nfidt,*) aext
      read (nfidt,*) roeo0
c
	nthd=int(nstep/iprint)+1
      theata0=theata0*pi/180.d0
c
      r=0.d0
      if ( betan .le. 0.25d0 ) then
          r=2.d0/( dsqrt(1.d0-4.d0*betan)*dsqrt(grav/xl) )
	    if ( tinc .ge. r ) then
	        write (nfthd, *) 'tinc violates stability condition'
		    stop 'tinc violates stability condition: it must be more smaller' 
	    end if
	end if 
c
	goto 777
c
c
  555 write(*, 5000)
 5000 format(' / flag required can not be found until end meet /')
      close (nfidt)
	stop 555
  999 write(*,9000)
 9000 format(' / Errors occur in reading / ')
      close (nfidt)
	stop 999
c
c
  777 close (nfidt)
      return
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   S O L V E _ M A I N     ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine solve_main
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
	tol_error=1.d-3
	mniteration=100
c
c
c     -------------------------------------------
c
c     / OPEN OUTPUT FILE /
c
      open (nfthd, file='Pendulum.thd')
      write (nfthd, 1000)
 1000 format('istep, time, disp, theata, vel, acc')
c
c
c     / INITIAL CONDITION /
c
      disp0=xl*theata0
	omega0=2.d0*pi*dsqrt(xl/grav)
	istep=0
      tduration=0.d0
c
      disp=disp0; vel=0.d0; acc=-grav*dsin(theata0)
	theata=theata0
	fextt=0.d0
      call cal_fext(tduration, disp, fextt, fextn); ffinc=0.d0
      call cal_frecover(disp, frecover, fextn)
      call cal_fresidual(disp,vel,acc,frecover,fextt,fresidual)
      call opt_position(istep, tduration, disp, theata,
     1                  vel, acc, fresidual, fextt)
c
c     / CALC. /
c
      do i=1, nstep
c
	    disp1=disp; vel1=vel; acc1=acc      ! memory
	    fextt1=fextt
c
	    istep=istep+1      ! update of step
          tduration=tduration+tinc    ! t+tinc
c                                     ! Newmark method
          niteration=0  !
          call cal_fext(tduration, disp, fextt, fextn)
	    ffinc=fextt-fextt1
          call cal_frecover(disp, frecover, fextn)
          call calc_Newmark(disp1, disp, vel, acc, ffinc,
     1                      dispinc, velinc, accinc,
     2                      fextt,fextn, fresidual, niteration)
          call cal_frecover(disp, frecover, fextn)
          call cal_fresidual(disp,vel,acc,frecover,fextt,fresidual)
          eps_error=dabs(fresidual)/(xmass*grav)
          goto 100
          DO WHILE ( (eps_error .gt. tol_error) .and.
     1               (niteration .le. mniteration) )
              call calc_Newmark(disp1, disp, vel, acc, ffinc,
     1                          dispinc, velinc, accinc,
     2                          fextt,fextn, fresidual, niteration)
              call cal_frecover(disp, frecover, fextn)
              call cal_fresidual(disp,vel,acc,frecover,fextt,fresidual)
              eps_error=dabs(fresidual)/(xmass*grav)
          END DO
  100     call opt_position(istep, tduration, disp, theata,
     1                      vel, acc, fresidual, fextt)
  	enddo
c
c
      close (nfthd)
c
c
      return
c
c
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   C A L _ F R E S I D U A L ******
c*                                                                   *
c*********************************************************************
c
c
      subroutine cal_fresidual(disp,vel,acc,frecover,fextt,fresidual)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
c
c
c     -------------------------------------------
c
      fresidual=0.d0
      finertia=xmass*acc
	fviscoc=viscc*vel
      fresidual=fextt-( finertia+fviscoc+frecover ) 
c
c
      return
c
c
	end
c
c
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   C A L _ F R E C O V E R ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine cal_frecover(disp, frecover, fextn)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
c
c
c     -------------------------------------------
c
      frecover=(xmass*grav-fextn)*dsin(disp/xl)
c
c
      return
c
c
	end
c
c
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   C A L _ F E X T         ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine cal_fext(tduration, disp, fextt, fextn)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
c
c
c     -------------------------------------------
c
	ddy=(roeo0*omega0)*(roeo0*omega0)*aext*dsin(roeo0*omega0*tduration+pi)
	fextt=xmass*ddy*dsin(disp/xl)
	fextn=xmass*ddy*dcos(disp/xl)
c
c	write(88,8000) tduration, ddy
c 8000 format(2e15.7)
c
c
      return
c
c
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   C A L C _ N e w m a r k ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine calc_Newmark(disp1, disp, vel, acc, ffinc,
     1                        dispinc, velinc, accinc,
     2                        fextt,fextn,fresidual, niteration)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
c
c     -------------------------------------------
c
c
      dispinc=0.d0; velinc=0.d0; accinc=0.d0
c
c ! appearance Kc
c
	stiff= (xmass*grav-fextn)/xl*dabs(dcos(disp1/xl)) 
      tk=( stiff+1.d0/betan/tinc/tinc*xmass+deltan/betan/tinc*viscc ) 
c
      if (niteration .eq. 0) then
          tf=ffinc
     1   + xmass*( 1.d0/betan/tinc*vel+0.5d0/betan*acc )
     2   + viscc*( deltan/betan*vel+(0.5d0*deltan/betan-1.d0)*tinc*acc )
	else
          tf=fresidual
      end if
c
      dispinc = tf / tk
c
      if ( niteration .eq. 0 ) then
          accinc=1.d0/betan/tinc/tinc
     1          *( dispinc-vel*tinc-0.5d0*tinc*tinc*acc )
          velinc=( acc+deltan*accinc )*tinc
          niteration=niteration+1
	else
          accinc=4.d0/tinc/tinc*dispinc
          velinc=2.d0/tinc*dispinc
          niteration=niteration+1
c	write(99,2000) niteration, disp, disp1, tk, tf,
c     1               fresidual, dispinc
c 2000 format(i5, 6e15.7)
      end if
c
      disp=disp+dispinc; vel=vel+velinc; acc=acc+accinc
c
c
      return
c
c
	end
c
c
c*********************************************************************
c*                                                                   *
c**********     S U B R O U T I N E   O P T _ P O S I T I O N ********
c*                                                                   *
c*********************************************************************
c
c
      subroutine opt_position(istep, tduration, disp, theata,
     1                        vel, acc, fresidual, fextt)
c
      implicit double precision(a-h, o-z)
      implicit integer (i-n)
c
c
	common /divtr/ xl, xmass, viscc, theata0, tinc, grav, omega0,
     1               aext, roeo0
	common /divti/ nstep, nthd
	common /neumr/ deltan, betan
	common /outpt/ iprint
	common /filen/ nfidt, nfthd
c
c
      pi = 4.d0 * datan(1.d0)
c
c     -------------------------------------------
c
      if (mod(istep, iprint) .ne. 0) goto 777
c
c
      theata=disp/xl
      write (nfthd, 1000) istep, tduration, disp, theata/pi*180.d0,
     1                    vel, acc
 1000 format(i10, 4(e15.7, ', '), e15.7)

c
c
  777 return
c
c
	end
c
c
